#!/usr/bin/ruby -Ku
# coding: utf-8
# $Id: galatea_config_maker.rb,v 1.15 2009/03/31 11:54:17 nishimoto Exp $
# (c) Takuya NISHIMOTO 

require 'fileutils'

class GalateaConfigMaker < ConfigMaker
  attr_reader :errors
  
  def initialize(file_array)
    puts "config #{file_array.join(' ')}"
    super
    @tmppath = param('tmppath') || (ENV['HOME'] + "/.galatea")
    puts "tmppath " + @tmppath
    @dmpath = param('dmpath')
    @enginepath = param('enginepath')
    
    @srm_gram_julian_path     = @enginepath + '/SRM/GramJulian'
    @srm_acoustic_model_path  = @enginepath + '/SRM/AcousticModel'
    @julius_server_runner_dir = @enginepath + '/SRM'
    @julius_server_runner_arg = "-C #{@tmppath}/julius.conf"
    @julius_server_runner_exe = param('julius.bin') || @enginepath + "/SRM/bin/julius"
    @srm_lang = 'ja'
    @srm_binhmm = 'hmmdefs,ptm,gid.binhmm'
    @dm_language = 'ja'
    @js_jar  = "/usr/share/java/js.jar"
    @gdm_jar = "../../bin/gdm.jar"
    #@classpath = @gdm_jar 
    @classpath = "#{@js_jar}:#{@gdm_jar}"
    @gdm_conf = "#{@tmppath}/gdm.conf"
    @am_commands_dm = "#{param('java.bin')} -cp #{@classpath} galatea.dialog.DialogManager -c #{@gdm_conf}"
    @am_commands_fsm = "cd #{@enginepath}/FSM/bin; ./fsm -C #{@tmppath}/fsm.conf"
    @am_commands_fs_mcl = "#{param('java.bin')} -cp #{@classpath} galatea.agent.BehaviorManager -c #{@gdm_conf}"
    @grammar_mkdfa_dir = "#{@enginepath}/SRM/bin"
    @grammar_mkdfa_exe = "mkdfa.pl"
    @gdm_grammar_work_dir = @tmppath
    @am_commands_am_mcl = "#{param('ruby.bin')} ./AM-MCL.rb -C #{@tmppath}/am_mcl.conf"
    @audiodev = param('audiodev') || "/dev/dsp"
    @errors = []
  end
  
  def set_dm_options(options, files)
    file = files.pop
    if file
      dm_arg = expand_path(file)
    else
      dm_arg = "#{param('dmpath')}/lib/default.vxml"
    end
    @am_commands_dm += " " + options.join(' ') + ' ' + dm_arg
  end

  def mkdir_tmppath
    unless File.exists?(@tmppath)
      FileUtils.mkdir_p @tmppath
    end
    File.chmod 0755, @tmppath
  end

  def set_lang(lang)
    @srm_lang = lang
    @dm_language = lang
    if lang == 'ja'
      @srm_binhmm = 'hmmdefs,ptm,gid.binhmm'
      @am_commands_ssm = "cd #{@enginepath}/SSM/gtalk ; ./gtalk -C #{@tmppath}/ssm.conf"
    else # 'en'
      @srm_binhmm = 'hmmdefs_mix16_gid.binhmm'
      @am_commands_ssm = "#{param('ruby.bin')} ./SSM-espeak.rb"
    end
  end  
  
  def make(file)
    begin
      ## for ruby 1.8.7 and 1.9.x
      #File.open("#{@tmppath}/#{file}", "w:ASCII-8BIT") do |outfile|
      #  str = File.open("#{@dmpath}/skel/#{file}.txt", "r:ASCII-8BIT").read
      #  outfile.write translate(str)
      #end
      ## for ruby 1.8.6
      File.open("#{@tmppath}/#{file}", "w") do |outfile|
        str = File.open("#{@dmpath}/skel/#{file}.txt", "r").read
        outfile.write translate(str)
     end
    rescue => error
      puts error.message
      exit
    end
  end
  
  def env_ok?
    files = [ 
      "#{@enginepath}/FSM/bin/fsm",
      "#{@enginepath}/SRM/bin/julius",
      "#{@enginepath}/SSM/gtalk/gtalk",
      "#{param('morphpath')}/chasen-2.4.1-istc/bin/chasen",
      "#{param('morphpath')}/chaone-1.3.2-istc/chaone",
    ]
    files.each do |f|
      unless File.stat(f).executable?
        @errors << "#{f} is not executable"    
      end
    end
    return false if @errors.size > 0
    true
  end
  
  def runner_cmd
    perl_bin = param('perl.bin')
    tmppath = param('tmppath')
    am_perl = "./AgentManager-gdm.pl"
    env = "export LANG=ja_JP.eucJP;export LC_ALL=ja_JP.eucJP;export PERL_BADLANG=0;export AUDIODEV=#{@audiodev}"
    "#{env}; cd #{param('dmpath')}/Modules; #{perl_bin} #{am_perl} -C #{tmppath}/am.conf"
  end

end
